==> musicman-0.8.6.tar.gz: musicman/musicman/musicman.cpp <==
/***************************************************************************
                          musicman.cpp  -  description
                             -------------------
    begin                : Tue May 27 21:25:35 BRT 2003
    copyright            : (C) 2003 by Avi Alkalay
    email                : avi@unix.sh
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <kaction.h>
#include <klocale.h>
#include <kconfig.h>
#include <kgenericfactory.h>
#include <kfileitem.h>
#include <kio/jobclasses.h>
#include <kio/job.h>
#include <qregexp.h>
#include <kfilemetainfo.h>
#include <qstringlist.h>
#include <kstandarddirs.h>
#include <dirent.h>


#include "musicman.h"
#include "musicindexgenerator.h"
#include "masstagger.h"
#include "customrenamer.h"
#include "musicfilenamelayout.h"
#include "musicinfo.h"


typedef KGenericFactory<MusicMan, KonqPopupMenu> MusicManFactory;
K_EXPORT_COMPONENT_FACTORY( libmusicman, MusicManFactory("musicman") );


MusicMan::MusicMan(KonqPopupMenu *popupmenu,const char *name,const QStringList &list) :
		KonqPopupMenuPlugin(popupmenu, name) {
	popup=popupmenu;
	miner=new FilenameMiner;
	menuMusicMan=new QPopupMenu(popup,"MusicMan");
	QString placeholder="musicman_placeholder";

	kdDebug(0) << "Musicman init: " << instance()->instanceName() <<endl;

	config=new KConfig("musicman");
	normalization.readConfig(config);

	actionDummy=new KAction(placeholder, 0, this,
		SLOT(slotPlaceholder()), actionCollection(), "Placeholder");
	addAction(actionDummy);
	addSeparator();

	connect(popup, SIGNAL(aboutToShow()),this,
		SLOT(slotAboutToShow()));

	setupActions();

	// Useless...... just to stop anoying compiler "unused parameter" warning
	list.count();
}







void MusicMan::setupActions() {
	/*
		Music Manager menu looks like this:

  ------------|Normalize Filename      |
  MusicMan   >|Set ID3 from Filename   |
  ------------|Advanced ID3 tagging    |
              |Create MP3 CD Cover     |
              |Rename space to _       +-------------+
              |Rename _ to space       |Layout 1.mp3 |
              |Rename Latin1 to UTF-8  |Layout 2.mp3 |
              |Set Filename from ID3  >|Layout 4.mp3 |
              +------------------------+Custom.mp3...|
                                       +-------------+
              
	*/

	// Setup base actions on the menu
  KAction *someAction;
	
	someAction=new KAction(i18n("Normalize Filename"),0,this,
		SLOT(slotNormalize()),actionCollection(),"Normalize");
	someAction->plug(menuMusicMan);

	someAction=new KAction(i18n("Rename space to _"),0,this,
		SLOT(slotToUnderline()),actionCollection(),"toUnderline");
	someAction->plug(menuMusicMan);

	someAction=new KAction(i18n("Rename _ to space"),0,this,
		SLOT(slotToSpace()),actionCollection(),"toSpace");
	someAction->plug(menuMusicMan);

	someAction=new KAction(i18n("Rename Latin1 to UTF-8"),0,this,
		SLOT(slotRenameLatin1ToUTF8()),actionCollection(),"Latin1toUTF8");
	someAction->setEnabled(false);

	someAction=new KAction(i18n("Set ID3 from Filename"),0,this,
		SLOT(slotID3FromFilename()),actionCollection(),"ID3FromFilename");
	someAction->plug(menuMusicMan);

	// kdDebug(0) << "Creating Filename Layout menus" << endl;

	// Build the menu for file-renaming based on ID3...
	menuFilenameFromID3=new QPopupMenu(menuMusicMan,i18n("Set Filename from ID3"));
	menuMusicMan->insertItem(i18n("Set Filename from ID3"),menuFilenameFromID3);

	int index=0;
	for (FilenameMiner::iterator it = miner->begin(); it != miner->end(); ++it ) {
		int id;
		id=menuFilenameFromID3->insertItem((*it).name,this,SLOT(slotFilenameFromID3(int)));
		menuFilenameFromID3->setItemParameter(id,index++);
		//kdDebug(0) << "Layout:" << (*it).name << endl;
	}
	someAction=new KAction(i18n("Custom ..."),0,this,
		SLOT(slotCustomRename()),actionCollection(),"CustomRename");
	someAction->plug(menuFilenameFromID3);


	someAction=new KAction(i18n("Advanced ID3 tagging ..."),0,this,
		SLOT(slotMassTag()),actionCollection(),"MassTag");
	someAction->plug(menuMusicMan);

	someAction=new KAction(i18n("Create MP3 CD Cover ..."),0,this,
		SLOT(slotIndex()),actionCollection(),"Index");
	someAction->plug(menuMusicMan);
}







MusicMan::~MusicMan() {
	// delete menuMusicMan; // deleted automatically by its parent
	// delete menuFilenameFromID3; // deleted automatically by its parent
	normalization.writeConfig(config);
}



void MusicMan::renameReplace(char from,char to) {
	KFileItemList selected;
	KFileItem *file;
	QString filename;
	QString _from,_to;

	_from=from;
	_to=to;

	selected=popup->fileItemList();

	for (file=selected.first();file;file=selected.next()) {
		KURL before=file->url();
		KURL after(before);

		
		filename=after.filename();
		filename.replace(_from,_to);
		after.setFileName(filename);
		
		kdDebug(126) << "before = " << before.url() << "\n";
		kdDebug(126) << "after  = " << after.url() << "\n\n";
		KIO::rename(before,after,false);
	}
}


/** Rename selected files from Latin1 to UTF-8 */
void MusicMan::slotRenameLatin1ToUTF8() {
	KFileItemList selected;
	KFileItem *file;
	QString filename;

	selected=popup->fileItemList();

	for (file=selected.first(); file; file=selected.next()) {
/*		KIO::UDSEntry fileInfo=file->entry();
		QString filename;

		KIO::UDSEntry::ConstIterator it = fileInfo.begin();
		for (; it != fileInfo.end(); ++it ) {
			switch ((*it).m_uds) {
				case KIO::UDS_NAME:
				
			}
        }  */
		
		KURL before=file->url();
		KURL after(before);
		kdDebug(126) << "Latin1: " << file->name().latin1() << "\n" << \
			"UTF-8: " << file->name().utf8() << "\n" << \
			"Pure: " << file->name() << "\n";
	}
}


void MusicMan::slotID3FromFilename() {
	KFileItem *file;
	KFileItemList selected=popup->fileItemList();

	for (file=selected.first(); file; file=selected.next()) {
		QString workString;
		MusicInfo musicInfo;

		// From the given filename layouts (in miner array), tries to extract some music info like author, title, etc.
		musicInfo=miner->mine(file->text());

		if (!musicInfo.isNull()) {
			KFileMetaInfo metaInfo=file->metaInfo();
			musicInfo.updateMetaInfo(metaInfo);
			metaInfo.applyChanges();
		}
		/*kdDebug(126) << "filename = " << filename << "\n" \
		             << "artist   = " << artist << "\n" \
		             << "title    = " << title << "\n" \
		             << "track    = " << track << "\n\n"; */
	}
}


/** Rename filename using ID3 values to match the layout specified by idx index in the miner (FilenameMiner). */
void MusicMan::slotFilenameFromID3(int idx) {
	KFileItem *file;
	KFileItemList selected=popup->fileItemList();

	for (file=selected.first();file;file=selected.next()) {
		KURL before=file->url();
		KURL after(before);
		MusicInfo minfo(*file); // Gets info from ID3 tags
		QString filename;

		filename=(*miner)[idx].getNewFilename(minfo,normalization);
		//kdDebug(0) << "Index: " << idx << endl << "Template: " << filename << endl;

		
		after.setFileName(filename);

		KIO::rename(before,after,false);
	}

}




void MusicMan::slotToUnderline() {
	renameReplace(' ','_');
}

void MusicMan::slotToSpace() {
	renameReplace('_',' ');
}




/** Starts creating the CD index, including HTML, Text, and the cover. */
void MusicMan::slotIndex() {
	KFileItem *file;
	KFileItemList selected=popup->fileItemList();

	for (file=selected.first();file;file=selected.next()) {
		if (file->isDir()) {
			indexer=new MusicIndexGenerator(file->url(),0L);
		} else {
			indexer=new MusicIndexGenerator(file->url().directory(),0L);
		}
	}
}



/** Rename the file to a standard layout. */
void MusicMan::slotNormalize() {
	KFileItem *file;
	KFileItemList selected=popup->fileItemList();

	for (file=selected.first();file;file=selected.next()) {
		KURL before=file->url();
		KURL after(before);
		QString filename;

		filename=after.filename();

		if (MusicFilenameLayout::normalize(filename,normalization)) {
			after.setFileName(filename);
			KIO::rename(before,after,false);
		}
	}
}


/** Remove the placeholder and put our submenus. */
void MusicMan::slotAboutToShow() {
	QString placeholder="musicman_placeholder";
	for(int i=popup->count(); i>=1; i--) {
		int id = popup->idAt( i );
		QString text = popup->text( id );
		if (text.contains(placeholder)) {
			popup->removeItem( id );
			popup->insertItem(i18n("Music Manager"),menuMusicMan,-1,i);
			delete actionDummy;
			break;
		}
	}
}

/** Void slot */
void MusicMan::slotPlaceholder(){
}

/** Calls the dialog for advanced ID3 tagging */
void MusicMan::slotMassTag() {
	KFileItemList selected=popup->fileItemList();

	massTagger=new MassTagger(selected,miner);
}



/** Calls the Custom File Renaming dialog, for filename renaming based on templates */
void MusicMan::slotCustomRename() {
	KFileItemList selected=popup->fileItemList();

	customRenamer=new CustomRenamer(selected,miner,&normalization,popup);
}

==> musicman-0.11.tar.gz: musicman/musicman/musicman.cpp <==
/***************************************************************************
                          musicman.cpp  -  description
                             -------------------
    begin                : Tue May 27 21:25:35 BRT 2003
    copyright            : (C) 2003 by Avi Alkalay
    email                : avi@unix.sh
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <kaction.h>
#include <klocale.h>
#include <kconfig.h>
#include <kgenericfactory.h>
#include <kfileitem.h>
#include <kio/jobclasses.h>
#include <kio/job.h>
#include <qregexp.h>
#include <kfilemetainfo.h>
#include <qstringlist.h>
#include <kstandarddirs.h>
#include <dirent.h>
#include <kiconloader.h> 


#include "musicman.h"
#include "musicindexgenerator.h"
#include "masstagger.h"
#include "customrenamer.h"
#include "musicfilenamelayout.h"
#include "musicinfo.h"


typedef KGenericFactory<MusicMan, KonqPopupMenu> MusicManFactory;
K_EXPORT_COMPONENT_FACTORY( libmusicman, MusicManFactory("musicman") );


MusicMan::MusicMan(KonqPopupMenu *popupmenu,const char *name,const QStringList &list) :
		KonqPopupMenuPlugin(popupmenu, name) {
	popup=popupmenu;
	miner=new FilenameMiner;
	menuMusicMan=new QPopupMenu(popup,"musicman");
	QString placeholder="musicman_placeholder";

	kdDebug(0) << "Musicman init: " << instance()->instanceName() <<endl;

	config=new KConfig("musicman");
	normalization.readConfig(config);
	//id3Encoding=KGlobal::charsets()->codecForName(config->readEntry("encoding"));

	actionDummy=new KAction(placeholder, 0, this,
		SLOT(slotPlaceholder()), actionCollection(), "Placeholder");
	addAction(actionDummy);
	addSeparator();

	connect(popup, SIGNAL(aboutToShow()),this,
		SLOT(slotAboutToShow()));

	setupActions();

	// Useless...... just to stop anoying compiler "unused parameter" warning
	list.count();
}







void MusicMan::setupActions() {
	/*
		Music Manager menu looks like this:

  ------------|Normalize Filename      |
  MusicMan   >|Set ID3 from Filename   |
  ------------|Advanced ID3 tagging    |
              |Create MP3 CD Cover     |
              |Rename space to _       +-------------+
              |Rename _ to space       |Layout 1.mp3 |
              |Rename Latin1 to UTF-8  |Layout 2.mp3 |
              |Set Filename from ID3  >|Layout 4.mp3 |
              +------------------------+Custom.mp3...|
                                       +-------------+
              
	*/

	// Setup base actions on the menu
	KAction *someAction;
	bool foldersOnly=onlyFolders();  
	
	someAction=new KAction(i18n("Normalize Filename"),0,this,
		SLOT(slotNormalize()),actionCollection(),"Normalize");
	someAction->plug(menuMusicMan);

	someAction=new KAction(i18n("Rename space to _"),0,this,
		SLOT(slotToUnderline()),actionCollection(),"toUnderline");
	someAction->plug(menuMusicMan);

	someAction=new KAction(i18n("Rename _ to space"),0,this,
		SLOT(slotToSpace()),actionCollection(),"toSpace");
	someAction->plug(menuMusicMan);

	//someAction=new KAction(i18n("Rename Latin1 to UTF-8"),0,this,
	//	SLOT(slotRenameLatin1ToUTF8()),actionCollection(),"Latin1toUTF8");
	//someAction->setEnabled(false);

	someAction=new KAction(i18n("Set ID3 from Filename"),0,this,
		SLOT(slotID3FromFilename()),actionCollection(),"ID3FromFilename");
	someAction->setEnabled(!foldersOnly);
	someAction->plug(menuMusicMan);

	// kdDebug(0) << "Creating Filename Layout menus" << endl;

	// Build the menu for file-renaming based on ID3...
	menuFilenameFromID3=new QPopupMenu(menuMusicMan,i18n("Set Filename from ID3"));
	menuMusicMan->setItemEnabled(menuMusicMan->insertItem(i18n("Set Filename from ID3"),menuFilenameFromID3),!foldersOnly);

	int index=0;
	for (FilenameMiner::iterator it = miner->begin(); it != miner->end(); ++it ) {
		int id;
		id=menuFilenameFromID3->insertItem((*it).name,this,SLOT(slotFilenameFromID3(int)));
		menuFilenameFromID3->setItemParameter(id,index++);
		//kdDebug(0) << "Layout:" << (*it).name << endl;
	}
	someAction=new KAction(i18n("Custom ..."),0,this,
		SLOT(slotCustomRename()),actionCollection(),"CustomRename");
	someAction->plug(menuFilenameFromID3);


	someAction=new KAction(i18n("Advanced ID3 tagging ..."),0,this,
		SLOT(slotMassTag()),actionCollection(),"MassTag");
	someAction->setEnabled(!foldersOnly);
	someAction->plug(menuMusicMan);

	someAction=new KAction(i18n("Create MP3 CD Cover ..."),0,this,
		SLOT(slotIndex()),actionCollection(),"Index");
	someAction->plug(menuMusicMan);
}







MusicMan::~MusicMan() {
	// delete menuMusicMan; // deleted automatically by its parent
	// delete menuFilenameFromID3; // deleted automatically by its parent

	kdDebug(0) << "Musicman finished: " << instance()->instanceName() <<endl;

	normalization.writeConfig(config);
}



void MusicMan::renameReplace(char from,char to) {
	KFileItemList selected;
	KFileItem *file;
	QString filename;
	QString _from,_to;

	_from=from;
	_to=to;

	selected=popup->fileItemList();

	for (file=selected.first();file;file=selected.next()) {
		KURL before=file->url();
		KURL after(before);

		
		filename=after.filename();
		filename.replace(_from,_to);
		after.setFileName(filename);
		
		kdDebug(126) << "before = " << before.url() << "\n";
		kdDebug(126) << "after  = " << after.url() << "\n\n";
		KIO::rename(before,after,false);
	}
}


/** Rename selected files from Latin1 to UTF-8 */
void MusicMan::slotRenameLatin1ToUTF8() {
	KFileItemList selected;
	KFileItem *file;
	QString filename;

	selected=popup->fileItemList();

	for (file=selected.first(); file; file=selected.next()) {
/*		KIO::UDSEntry fileInfo=file->entry();
		QString filename;

		KIO::UDSEntry::ConstIterator it = fileInfo.begin();
		for (; it != fileInfo.end(); ++it ) {
			switch ((*it).m_uds) {
				case KIO::UDS_NAME:
				
			}
        }  */
		
		KURL before=file->url();
		KURL after(before);
		kdDebug(126) << "Latin1: " << file->name().latin1() << "\n" << \
			"UTF-8: " << file->name().utf8() << "\n" << \
			"Pure: " << file->name() << "\n";
	}
}


void MusicMan::slotID3FromFilename() {
	KFileItem *file;
	KFileItemList selected=popup->fileItemList();

	for (file=selected.first(); file; file=selected.next()) {
		QString workString;
		MusicInfo musicInfo;

		// From the given filename layouts (in miner array), tries to extract some music info like author, title, etc.
		musicInfo=miner->mine(file->text());

		if (!musicInfo.isNull()) {
			KFileMetaInfo metaInfo=file->metaInfo();
			if (!metaInfo.isEmpty()) {
				musicInfo.updateMetaInfo(metaInfo);
				metaInfo.applyChanges();
			}
		}
		/*kdDebug(126) << "filename = " << filename << "\n" \
		             << "artist   = " << artist << "\n" \
		             << "title    = " << title << "\n" \
		             << "track    = " << track << "\n\n"; */
	}
}


/** Rename filename using ID3 values to match the layout specified by idx index in the miner (FilenameMiner). */
void MusicMan::slotFilenameFromID3(int idx) {
	KFileItem *file;
	KFileItemList selected=popup->fileItemList();

	for (file=selected.first();file;file=selected.next()) {
		KURL before=file->url();
		KURL after(before);
		MusicInfo minfo(*file); // Gets info from ID3 tags
		QString filename;

		filename=(*miner)[idx].getNewFilename(minfo,normalization);
		//kdDebug(0) << "Index: " << idx << endl << "Template: " << filename << endl;

		
		after.setFileName(filename);

		KIO::rename(before,after,false);
	}

}




void MusicMan::slotToUnderline() {
	renameReplace(' ','_');
}

void MusicMan::slotToSpace() {
	renameReplace('_',' ');
}




/** Starts creating the CD index, including HTML, Text, and the cover. */
void MusicMan::slotIndex() {
	KFileItem *file;
	KFileItemList selected=popup->fileItemList();

	for (file=selected.first();file;file=selected.next()) {
		if (file->isDir()) {
			indexer=new MusicIndexGenerator(file->url(),0L);
		} else {
			indexer=new MusicIndexGenerator(file->url().directory(),0L);
		}
	}
}



/** Rename the file to a standard layout. */
void MusicMan::slotNormalize() {
	KFileItem *file;
	KFileItemList selected=popup->fileItemList();

	for (file=selected.first();file;file=selected.next()) {
		KURL before=file->url();
		KURL after(before);
		QString filename;

		filename=after.filename();

		if (MusicFilenameLayout::normalize(filename,normalization)) {
			after.setFileName(filename);
			KIO::rename(before,after,false);
		}
	}
}


/** Remove the placeholder and put our submenus. */
void MusicMan::slotAboutToShow() {
	KIconLoader iconLoader;
	QString placeholder="musicman_placeholder";
	for(int i=popup->count(); i>=1; i--) {
		int id = popup->idAt( i );
		QString text = popup->text( id );
		if (text.contains(placeholder)) {
			popup->removeItem( id );
			popup->insertItem(iconLoader.loadIcon("musicman",KIcon::Small),i18n("Music Manager"),menuMusicMan,-1,i);
			delete actionDummy;
			break;
		}
	}
}

/** Void slot */
void MusicMan::slotPlaceholder(){
}

/** Calls the dialog for advanced ID3 tagging */
void MusicMan::slotMassTag() {
	KFileItemList selected=popup->fileItemList();

	massTagger=new MassTagger(selected,miner);
}



/** Calls the Custom File Renaming dialog, for filename renaming based on templates */
void MusicMan::slotCustomRename() {
	KFileItemList selected=popup->fileItemList();

	customRenamer=new CustomRenamer(selected,miner,&normalization,popup);
}



/** @returns false if there is files selected
@returns true if there is only folders selected */
bool MusicMan::onlyFolders() {
	KFileItem *file;
	KFileItemList selected=popup->fileItemList();

	for (file=selected.first();file;file=selected.next()) {
		if (file->isFile()) return false;
	}
	return true;
}
